//+--------------------------------------------------------------------------------------------+
//|                                                                             DrawOrders.mq4 |
//|                                                             Copyright  2009, Serega Lykov |
//|                                                                 http://mtexperts.narod.ru/ |
//+--------------------------------------------------------------------------------------------+

#property copyright "Copyright  2009, Serega Lykov"
#property link      "http://mtexperts.narod.ru/"

//----   --------------------------------------------------------------------+
#property indicator_chart_window

//----   ----------------------------------------------------------------------+
extern int    MagicNumber      = 0;
extern color  BuyOrdersColor   = Blue;
extern color  SellOrdersColor  = Red;
extern bool   DeleteObjects    = false;

//-----   -----------------------------------------------------------------+
static int      CodeArrowOpen       = 1;
static int      CodeArrowClose      = 3;
static string   symbol;
static string   short_name          = "DrawOrders";
static string   obj_signature       = "DO";
static string   obj_signature_open  = "_Open ";
static string   obj_signature_close = "_Close ";
static string   obj_signature_trade = "_Trade ";
static string   err_message_1       = "   ,  = ";
static string   sign_open;
static string   sign_close;
static string   sign_trade;
static int      length_sign_open;

//---------------------------------------------------------------------------------------------+
//----   ---------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int init()
  {
   //----   ----------------------------------------------------------------+
   symbol     = Symbol();
   sign_open  = StringConcatenate(obj_signature,obj_signature_open);
   sign_close = StringConcatenate(obj_signature,obj_signature_close);
   sign_trade = StringConcatenate(obj_signature,obj_signature_trade);
   length_sign_open = StringLen(sign_open);
   //----  ""   ----------------------------------------------+
   IndicatorShortName(short_name);
   //----   -----------------------------------------------------------------+
   return(0);
  }

//---------------------------------------------------------------------------------------------+
//----   -------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int deinit()
  {
   return(0);
  }

//---------------------------------------------------------------------------------------------+
//---- DrawOrders -----------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int start()
  {
   if(DeleteObjects)
     {
      DeleteObjects(obj_signature);
      return(0);
     }
   double spread = NormalizeDouble(MarketInfo(symbol,MODE_SPREAD)*Point,Digits);
   int total = OrdersTotal() - 1;
   for(int i=total; i>=0; i--)
     {
      if(!OrderSelect(i,SELECT_BY_POS,MODE_TRADES))
        {
         Print(StringConcatenate(err_message_1,DoubleToStr(GetLastError(),0)));
         continue;
        }
      if(MagicNumber >= 0) if(OrderMagicNumber() != MagicNumber) continue;
      if(OrderSymbol() != symbol) continue;
      string str_order_ticket = DoubleToStr(OrderTicket(),0);
      int order_type = OrderType();
      string obj_name = StringConcatenate(sign_open,str_order_ticket);
      if((order_type % 2) == OP_BUY) double order_open_price = NormalizeDouble(OrderOpenPrice() - spread,Digits);
      else order_open_price = NormalizeDouble(OrderOpenPrice(),Digits);
      if(ObjectFind(obj_name) < 0)
        {
         ObjectCreate(obj_name,OBJ_ARROW,0,OrderOpenTime(),order_open_price);
         ObjectSet(obj_name,OBJPROP_STYLE,STYLE_SOLID);
         if((order_type % 2) == OP_BUY) ObjectSet(obj_name,OBJPROP_COLOR,BuyOrdersColor);
         else ObjectSet(obj_name,OBJPROP_COLOR,SellOrdersColor);
         ObjectSet(obj_name,OBJPROP_ARROWCODE,CodeArrowOpen);
        }
      else
        {
         double obj_price1  = NormalizeDouble(ObjectGet(obj_name,OBJPROP_PRICE1),Digits);
         datetime obj_time1 = ObjectGet(obj_name,OBJPROP_TIME1);
         if(obj_price1 != order_open_price || obj_time1 != OrderOpenTime()) ObjectMove(obj_name,0,OrderOpenTime(),order_open_price);
        }
     }
   int obj_total = ObjectsTotal() - 1;
   for(i=obj_total; i>=0; i--)
     {
      obj_name = ObjectName(i);
      if(ObjectType(obj_name) != OBJ_ARROW) continue;
      if(StringFind(obj_name,sign_open) != 0) continue;
      int order_ticket = StrToInteger(StringSubstr(obj_name,length_sign_open));
      if(!OrderSelect(order_ticket,SELECT_BY_TICKET)) continue;
      if(OrderCloseTime() == 0) continue;
      order_type = OrderType();
      if((order_type % 2) == OP_BUY)
        {
         order_open_price         = NormalizeDouble(OrderOpenPrice() - spread,Digits);
         double order_close_price = NormalizeDouble(OrderClosePrice(),Digits);
        }
      else
        {
         order_open_price  = NormalizeDouble(OrderOpenPrice(),Digits);
         order_close_price = NormalizeDouble(OrderClosePrice() - spread,Digits);
        }
      obj_name = StringConcatenate(sign_close,DoubleToStr(order_ticket,0));
      if(ObjectFind(obj_name) < 0)
        {
         ObjectCreate(obj_name,OBJ_ARROW,0,OrderCloseTime(),order_close_price);
         ObjectSet(obj_name,OBJPROP_STYLE,STYLE_SOLID);
         if((order_type % 2) == OP_BUY) ObjectSet(obj_name,OBJPROP_COLOR,BuyOrdersColor);
         else ObjectSet(obj_name,OBJPROP_COLOR,SellOrdersColor);
         ObjectSet(obj_name,OBJPROP_ARROWCODE,CodeArrowClose);
        }
      else
        {
         obj_price1  = NormalizeDouble(ObjectGet(obj_name,OBJPROP_PRICE1),Digits);
         obj_time1 = ObjectGet(obj_name,OBJPROP_TIME1);
         if(obj_price1 != order_close_price || obj_time1 != OrderCloseTime()) ObjectMove(obj_name,0,OrderCloseTime(),order_close_price);
        }
      obj_name = StringConcatenate(sign_trade,DoubleToStr(order_ticket,0));
      if(ObjectFind(obj_name) < 0)
        {
         ObjectCreate(obj_name,OBJ_TREND,0,OrderOpenTime(),order_open_price,OrderCloseTime(),order_close_price);
         ObjectSet(obj_name,OBJPROP_STYLE,STYLE_DOT);
         if((order_type % 2) == OP_BUY) ObjectSet(obj_name,OBJPROP_COLOR,BuyOrdersColor);
         else ObjectSet(obj_name,OBJPROP_COLOR,SellOrdersColor);
         ObjectSet(obj_name,OBJPROP_RAY,false);
        }
      else
        {
         obj_price1         = NormalizeDouble(ObjectGet(obj_name,OBJPROP_PRICE1),Digits);
         obj_time1          = ObjectGet(obj_name,OBJPROP_TIME1);
         double obj_price2  = NormalizeDouble(ObjectGet(obj_name,OBJPROP_PRICE2),Digits);
         datetime obj_time2 = ObjectGet(obj_name,OBJPROP_TIME2);
         if(obj_price1 != order_open_price  || obj_time1 != OrderOpenTime())  ObjectMove(obj_name,0,OrderOpenTime(),order_open_price);
         if(obj_price2 != order_close_price || obj_time2 != OrderCloseTime()) ObjectMove(obj_name,1,OrderCloseTime(),order_close_price);
        }
     }
   return(0);
  }

//---------------------------------------------------------------------------------------------+
//---- DeleteObjects --------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
void DeleteObjects(string obj_sign)
  {
   int obj_total = ObjectsTotal() - 1;
   for(int i=obj_total; i>=0; i--)
     {
      string obj_name = ObjectName(i);
      int obj_type = ObjectType(obj_name);
      if(obj_type != OBJ_ARROW && obj_type != OBJ_TREND) continue;
      if(StringFind(obj_name,obj_sign,0) != 0) continue;
      ObjectDelete(obj_name);
     }
  }

//---------------------------------------------------------------------------------------------+